--------------------------------
-------Resource Aware ML--------
--------------------------------
-- File:
--   Nat.hs
--
-- Author:
--   Jan Hoffmann (2011)
--
-- Description:
--   Hardcoding operations for natural numbers.


module Nat where

import MultivariatePol
import Util
import qualified Data.Map as M



multCoef :: TIndices -> TIndices -> TIndices -> Integer
-- Recursive computation of the coefficients A(i,j,k) in
--   sum_i,j (A(i,j,k)*binom(n,i)*binom(m,j)) = binom(n*m,k)
--
-- Following Riordan and Stein, in "Arrangements on Chessboards"
-- (Journal of Combinatorial Theory, Series A, 12 72-80, 1972)

multCoef (TIList is) (TIList js) (TIList ks) = multC is js ks

multC is js ks =
   case (is,js,ks) of
      ([],[],[]   ) -> 1
      ([],_,_    ) -> 0
      (_,[],_    ) -> 0
      (_,_,[]    ) -> 0
      (i:is',j:js',k:ks') -> let l = toInteger . length in
           ( (l is)*(l js)*
             ( (multC is js ks')
             + (multC is' js ks')
             + (multC is js' ks')
             + (multC is' js' ks')
             )
             - (l ks') * (multC is js ks') 
           ) `div` (l ks)




-- -- For tests

-- mk 0 = []
-- mk (n+1) = TIUnit:mk n

-- binomStr k = "expand (binomial(n*m,"++(show k)++") - ( " 
--             ++ (tail . concat)  [ "+ " ++ show (multC (mk i) (mk j) (mk k))
--                                        ++" * binomial(n,"++ (show i) 
--                                        ++")*binomial(m," ++ (show j)++")"  | i <- [0..k], j <- [0..k] ]
--                ++ "));"

